@@ -1,5 +1,5 @@
 # Copyright © 2008-2011 Raphaël Hertzog <hertzog@debian.org>
-# Copyright © 2008-2015 Guillem Jover <guillem@debian.org>
+# Copyright © 2008-2025 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,8 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::Source::Package;
-
 =encoding utf8
 
 =head1 NAME
@@ -24,17 +22,17 @@ Dpkg::Source::Package - manipulate Debian source packages
 
 =head1 DESCRIPTION
 
-This module provides an object that can manipulate Debian source
+This module provides a class that can manipulate Debian source
 packages. While it supports both the extraction and the creation
 of source packages, the only API that is officially supported
 is the one that supports the extraction of the source package.
 
 =cut
 
-use strict;
-use warnings;
+package Dpkg::Source::Package 2.04;
+
+use v5.36;
 
-our $VERSION = '1.03';
 our @EXPORT_OK = qw(
     get_default_diff_ignore_regex
     set_default_diff_ignore_regex
@@ -42,9 +40,10 @@ our @EXPORT_OK = qw(
 );
 
 use Exporter qw(import);
-use POSIX qw(:errno_h :sys_wait_h);
 use Carp;
+use File::Copy qw(cp);
 use File::Basename;
+use File::Spec;
 
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
@@ -52,40 +51,34 @@ use Dpkg::Control;
 use Dpkg::Checksums;
 use Dpkg::Version;
 use Dpkg::Compression;
-use Dpkg::Exit qw(run_exit_handlers);
-use Dpkg::Path qw(check_files_are_the_same find_command);
-use Dpkg::IPC;
+use Dpkg::Path qw(check_files_are_the_same check_directory_traversal);
 use Dpkg::Vendor qw(run_vendor_hook);
 use Dpkg::Source::Format;
+use Dpkg::OpenPGP;
+use Dpkg::OpenPGP::ErrorCodes;
 
 my $diff_ignore_default_regex = '
-# Ignore general backup files
+# Ignore general backup files.
 (?:^|/).*~$|
-# Ignore emacs recovery files
+# Ignore emacs recovery files.
 (?:^|/)\.#.*$|
-# Ignore vi swap files
+# Ignore vi swap files.
 (?:^|/)\..*\.sw.$|
-# Ignore baz-style junk files or directories
+# Ignore baz-style junk files or directories.
 (?:^|/),,.*(?:$|/.*$)|
-# File-names that should be ignored (never directories)
+# File-names that should be ignored (never directories).
 (?:^|/)(?:DEADJOE|\.arch-inventory|\.(?:bzr|cvs|hg|git|mtn-)ignore)$|
-# File or directory names that should be ignored
+# File or directory names that should be ignored.
 (?:^|/)(?:CVS|RCS|\.deps|\{arch\}|\.arch-ids|\.svn|
 \.hg(?:tags|sigs)?|_darcs|\.git(?:attributes|modules|review)?|
 \.mailmap|\.shelf|_MTN|\.be|\.bzr(?:\.backup|tags)?)(?:$|/.*$)
 ';
-# Take out comments and newlines
+# Take out comments and newlines.
 $diff_ignore_default_regex =~ s/^#.*$//mg;
 $diff_ignore_default_regex =~ s/\n//sg;
 
-# Public variables
-# XXX: Backwards compatibility, stop exporting on VERSION 2.00.
-## no critic (Variables::ProhibitPackageVars)
-our $diff_ignore_default_regexp;
-*diff_ignore_default_regexp = \$diff_ignore_default_regex;
-
 no warnings 'qw'; ## no critic (TestingAndDebugging::ProhibitNoWarnings)
-our @tar_ignore_default_pattern = qw(
+my @tar_ignore_default_pattern = qw(
 *.a
 *.la
 *.o
@@ -169,36 +162,58 @@ sub get_default_tar_ignore_pattern {
 
 =item $p = Dpkg::Source::Package->new(%opts, options => {})
 
-Creates a new object corresponding to a source package. When the key
-B<filename> is set to a F<.dsc> file, it will be used to initialize the
-source package with its description. Otherwise if the B<format> key is
-set to a valid value, the object will be initialized for that format
-(since dpkg 1.19.3).
+Creates a new object corresponding to a source package.
 
-The B<options> key is a hash ref which supports the following options:
+Options:
+
+=over
+
+=item B<filename>
+
+When set to a F<.dsc> file, it will be used to initialize the
+source package with its description.
+
+=item B<format>
+
+If set to a valid value, and no filename has been specified,
+the object will be initialized for that format (since dpkg 1.19.3).
+
+=item B<options>
+
+A hash ref which supports the following source package format options:
 
 =over 8
 
-=item skip_debianization
+=item B<skip_debianization>
 
 If set to 1, do not apply Debian changes on the extracted source package.
 
-=item skip_patches
+=item B<skip_patches>
 
 If set to 1, do not apply Debian-specific patches. This options is
 specific for source packages using format "2.0" and "3.0 (quilt)".
 
-=item require_valid_signature
+=item B<certs>
+
+An array ref with a list of certificate keyrings to use for signature
+verification.
+
+=item B<use_vendor_certs>
+
+If set to 0, the check_signature() method will not use vendor specific
+keyrings, only user supplied ones.
+
+=item B<require_valid_signature>
 
 If set to 1, the check_signature() method will be stricter and will error
 out if the signature can't be verified.
 
-=item require_strong_checksums
+=item B<require_strong_checksums>
 
 If set to 1, the check_checksums() method will be stricter and will error
 out if there is no strong checksum.
 
-=item copy_orig_tarballs
+=item B<copy_orig_tarballs>
 
 If set to 1, the extraction will copy the upstream tarballs next the
 target directory. This is useful if you want to be able to rebuild the
@@ -206,37 +221,50 @@ source package after its extraction.
 
 =back
 
+=back
+
 =cut
 
-# Object methods
 sub new {
-    my ($this, %args) = @_;
+    my ($this, %opts) = @_;
     my $class = ref($this) || $this;
     my $self = {
-        fields => Dpkg::Control->new(type => CTRL_PKG_SRC),
+        fields => Dpkg::Control->new(type => CTRL_DSC),
         format => Dpkg::Source::Format->new(),
         options => {},
         checksums => Dpkg::Checksums->new(),
+        openpgp => Dpkg::OpenPGP->new(
+            needs => {
+                api => 'verify',
+            },
+        ),
     };
     bless $self, $class;
-    if (exists $args{options}) {
-        $self->{options} = $args{options};
+    if (exists $opts{options}) {
+        $self->{options} = $opts{options};
     }
-    if (exists $args{filename}) {
-        $self->initialize($args{filename});
+    if (exists $opts{filename}) {
+        $self->initialize($opts{filename});
         $self->init_options();
-    } elsif ($args{format}) {
-        $self->{fields}{Format} = $args{format};
+    } elsif ($opts{format}) {
+        $self->{fields}{Format} = $opts{format};
         $self->upgrade_object_type(0);
         $self->init_options();
     }
+
+    if ($self->{options}{require_valid_signature}) {
+        $self->{report_verify} = \&error;
+    } else {
+        $self->{report_verify} = \&warning;
+    }
+
     return $self;
 }
 
 sub init_options {
     my $self = shift;
-    # Use full ignore list by default
-    # note: this function is not called by V1 packages
+    # Use full ignore list by default.
+    # Note: This function is not called by V1 packages.
     $self->{options}{diff_ignore_regex} ||= $diff_ignore_default_regex;
     $self->{options}{diff_ignore_regex} .= '|(?:^|/)debian/source/local-.*$';
     $self->{options}{diff_ignore_regex} .= '|(?:^|/)debian/files(?:\.new)?$';
@@ -247,20 +275,27 @@ sub init_options {
         $self->{options}{tar_ignore} = [ @tar_ignore_default_pattern ];
     }
     push @{$self->{options}{tar_ignore}},
-         'debian/source/local-options',
-         'debian/source/local-patch-header',
-         'debian/files',
-         'debian/files.new';
-    # Skip debianization while specific to some formats has an impact
-    # on code common to all formats
+        'debian/source/local-options',
+        'debian/source/local-patch-header',
+        'debian/files',
+        'debian/files.new';
+    $self->{options}{copy_orig_tarballs} //= 0;
+
+    # Skip debianization while specific to some formats has an impact.
+    # on code common to all formats.
     $self->{options}{skip_debianization} //= 0;
+    $self->{options}{skip_patches} //= 0;
+
+    # Set default validation checks.
+    $self->{options}{certs} //= [];
+    $self->{options}{use_vendor_certs} //= 1;
+    $self->{options}{require_valid_signature} //= 0;
+    $self->{options}{require_strong_checksums} //= 0;
 
     # Set default compressor for new formats.
     $self->{options}{compression} //= 'xz';
-    $self->{options}{comp_level} //= compression_get_property($self->{options}{compression},
-                                                              'default_level');
-    $self->{options}{comp_ext} //= compression_get_property($self->{options}{compression},
-                                                            'file_ext');
+    $self->{options}{comp_level} //= compression_get_level($self->{options}{compression});
+    $self->{options}{comp_ext} //= compression_get_file_extension($self->{options}{compression});
 }
 
 sub initialize {
@@ -270,7 +305,7 @@ sub initialize {
     $self->{basedir} = $dir || './';
     $self->{filename} = $fn;
 
-    # Read the fields
+    # Read the fields.
     my $fields = $self->{fields};
     $fields->load($filename);
     $self->{is_signed} = $fields->get_option('is_pgp_signed');
@@ -296,7 +331,6 @@ sub upgrade_object_type {
     my $module = "Dpkg::Source::Package::V$major";
     $module .= '::' . ucfirst $variant if defined $variant;
     eval qq{
-        pop \@INC if \$INC[-1] eq '.';
         require $module;
         \$minor = \$${module}::CURRENT_MINOR_VERSION;
     };
@@ -321,7 +355,7 @@ Returns the filename of the DSC file.
 
 sub get_filename {
     my $self = shift;
-    return $self->{basedir} . $self->{filename};
+    return File::Spec->catfile($self->{basedir}, $self->{filename});
 }
 
 =item $p->get_files()
@@ -353,7 +387,7 @@ sub check_checksums {
     my $checksums = $self->{checksums};
     my $warn_on_weak = 0;
 
-    # add_from_file verify the checksums if they are already existing
+    # The add_from_file() call verifies the checksums if they already exist.
     foreach my $file ($checksums->get_files()) {
         if (not $checksums->has_strong_checksums($file)) {
             if ($self->{options}{require_strong_checksums}) {
@@ -362,7 +396,8 @@ sub check_checksums {
                 $warn_on_weak = 1;
             }
         }
-	$checksums->add_from_file($self->{basedir} . $file, key => $file);
+        my $pathname = File::Spec->catfile($self->{basedir}, $file);
+        $checksums->add_from_file($pathname, key => $file);
     }
 
     warning(g_('source package uses only weak checksums')) if $warn_on_weak;
@@ -376,10 +411,28 @@ sub get_basename {
               'Source', 'Version');
     }
     my $v = Dpkg::Version->new($f->{'Version'});
-    my $vs = $v->as_string(omit_epoch => 1, omit_revision => !$with_revision);
+    my $vs = $v->as_string(
+        omit_epoch => 1,
+        omit_revision => ! $with_revision,
+    );
     return $f->{'Source'} . '_' . $vs;
 }
 
+=item $p->get_basedirname()
+
+Returns the default base directory name for the package.
+
+=cut
+
+sub get_basedirname {
+    my $self = shift;
+
+    my $dirname = $self->get_basename();
+    $dirname =~ s/_/-/;
+
+    return $dirname;
+}
+
 sub find_original_tarballs {
     my ($self, %opts) = @_;
     $opts{extension} //= compression_get_file_extension_regex();
@@ -390,17 +443,77 @@ sub find_original_tarballs {
     foreach my $dir ('.', $self->{basedir}, $self->{options}{origtardir}) {
         next unless defined($dir) and -d $dir;
         opendir(my $dir_dh, $dir) or syserr(g_('cannot opendir %s'), $dir);
-        push @tar, map { "$dir/$_" } grep {
-		($opts{include_main} and
-		 /^\Q$basename\E\.orig\.tar\.$opts{extension}$/) or
-		($opts{include_supplementary} and
-		 /^\Q$basename\E\.orig-[[:alnum:]-]+\.tar\.$opts{extension}$/)
-	    } readdir($dir_dh);
+        push @tar, map { File::Spec->catfile($dir, $_) } grep {
+                ($opts{include_main} and
+                 /^\Q$basename\E\.orig\.tar\.$opts{extension}$/) or
+                ($opts{include_supplementary} and
+                 /^\Q$basename\E\.orig-[[:alnum:]-]+\.tar\.$opts{extension}$/)
+            } readdir($dir_dh);
         closedir($dir_dh);
     }
     return @tar;
 }
 
+=item $p->get_upstream_signing_key($dir)
+
+Get the filename for the upstream key.
+
+=cut
+
+sub get_upstream_signing_key {
+    my ($self, $dir) = @_;
+
+    return "$dir/debian/upstream/signing-key.asc";
+}
+
+=item $p->armor_original_tarball_signature($bin, $asc)
+
+Convert a signature from binary to ASCII armored form. If the signature file
+does not exist, it is a no-op. If the signature file is already ASCII armored
+then simply copy it, otherwise convert it from binary to ASCII armored form.
+
+=cut
+
+sub armor_original_tarball_signature {
+    my ($self, $bin, $asc) = @_;
+
+    if (-e $bin) {
+        return $self->{openpgp}->armor('SIGNATURE', $bin, $asc);
+    }
+
+    return;
+}
+
+=item $p->check_original_tarball_signature($dir, @asc)
+
+Verify the original upstream tarball signatures @asc using the upstream
+public keys. It requires the origin upstream tarballs, their signatures
+and the upstream signing key, as found in an unpacked source tree $dir.
+If any inconsistency is discovered, it immediately errors out.
+
+=cut
+
+sub check_original_tarball_signature {
+    my ($self, $dir, @asc) = @_;
+
+    my $upstream_key = $self->get_upstream_signing_key($dir);
+    if (not -e $upstream_key) {
+        warning(g_('upstream tarball signatures but no upstream signing key'));
+        return;
+    }
+
+    foreach my $asc (@asc) {
+        my $datafile = $asc =~ s/\.asc$//r;
+
+        info(g_('verifying %s'), $asc);
+        my $rc = $self->{openpgp}->verify($datafile, $asc, $upstream_key);
+        if ($rc) {
+            $self->{report_verify}->(g_('cannot verify upstream tarball signature for %s: %s'),
+                                     $datafile, openpgp_errorcode_to_string($rc));
+        }
+    }
+}
+
 =item $bool = $p->is_signed()
 
 Returns 1 if the DSC files contains an embedded OpenPGP signature.
@@ -426,51 +539,38 @@ then any problem will result in a fatal error.
 sub check_signature {
     my $self = shift;
     my $dsc = $self->get_filename();
-    my @exec;
+    my @certs;
 
-    if (find_command('gpgv2')) {
-        push @exec, 'gpgv2';
-    } elsif (find_command('gpgv')) {
-        push @exec, 'gpgv';
-    } elsif (find_command('gpg2')) {
-        push @exec, 'gpg2', '--no-default-keyring', '-q', '--verify';
-    } elsif (find_command('gpg')) {
-        push @exec, 'gpg', '--no-default-keyring', '-q', '--verify';
-    }
-    if (scalar(@exec)) {
-        if (length $ENV{HOME} and -r "$ENV{HOME}/.gnupg/trustedkeys.gpg") {
-            push @exec, '--keyring', "$ENV{HOME}/.gnupg/trustedkeys.gpg";
+    info(g_('verifying %s'), $dsc);
+
+    # User specified signer certificates, otherwise fallback to use the
+    # trusted keyrings.
+    if (@{$self->{options}{certs}}) {
+        push @certs, @{$self->{options}{certs}};
+    } else {
+        foreach my $keyring ($self->{openpgp}->get_trusted_keyrings()) {
+            push @certs, $keyring;
+            warning(g_('using implicit trusted keyring %s is deprecated; ' .
+                       'use --signer-cert with an OpenPGP keyring instead'),
+                    $keyring);
         }
+    }
+
+    if ($self->{options}{use_vendor_certs}) {
         foreach my $vendor_keyring (run_vendor_hook('package-keyrings')) {
             if (-r $vendor_keyring) {
-                push @exec, '--keyring', $vendor_keyring;
+                push @certs, $vendor_keyring;
+                info(g_('using keyring %s'), $vendor_keyring);
+            } else {
+                info(g_('skipping absent keyring %s'), $vendor_keyring);
             }
         }
-        push @exec, $dsc;
+    }
 
-        my ($stdout, $stderr);
-        spawn(exec => \@exec, wait_child => 1, nocheck => 1,
-              to_string => \$stdout, error_to_string => \$stderr,
-              timeout => 10);
-        if (WIFEXITED($?)) {
-            my $gpg_status = WEXITSTATUS($?);
-            print { *STDERR } "$stdout$stderr" if $gpg_status;
-            if ($gpg_status == 1 or ($gpg_status &&
-                $self->{options}{require_valid_signature}))
-            {
-                error(g_('failed to verify signature on %s'), $dsc);
-            } elsif ($gpg_status) {
-                warning(g_('failed to verify signature on %s'), $dsc);
-            }
-        } else {
-            subprocerr("@exec");
-        }
-    } else {
-        if ($self->{options}{require_valid_signature}) {
-            error(g_('cannot verify signature on %s since GnuPG is not installed'), $dsc);
-        } else {
-            warning(g_('cannot verify signature on %s since GnuPG is not installed'), $dsc);
-        }
+    my $rc = $self->{openpgp}->inline_verify($dsc, undef, @certs);
+    if ($rc) {
+        $self->{report_verify}->(g_('cannot verify inline signature for %s: %s'),
+                                 $dsc, openpgp_errorcode_to_string($rc));
     }
 }
 
@@ -511,57 +611,45 @@ sub extract {
         }
     }
 
-    # Copy orig tarballs
+    # Copy orig tarballs.
     if ($self->{options}{copy_orig_tarballs}) {
         my $basename = $self->get_basename();
         my ($dirname, $destdir) = fileparse($newdirectory);
         $destdir ||= './';
-	my $ext = compression_get_file_extension_regex();
+        my $ext = compression_get_file_extension_regex();
         foreach my $orig (grep { /^\Q$basename\E\.orig(-[[:alnum:]-]+)?\.tar\.$ext$/ }
                           $self->get_files())
         {
             my $src = File::Spec->catfile($self->{basedir}, $orig);
             my $dst = File::Spec->catfile($destdir, $orig);
             if (not check_files_are_the_same($src, $dst, 1)) {
-                system('cp', '--', $src, $dst);
-                subprocerr("cp $src to $dst") if $?;
+                cp($src, $dst)
+                    or syserr(g_('cannot copy %s to %s'), $src, $dst);
             }
         }
     }
 
-    # Try extract
-    eval { $self->do_extract($newdirectory) };
-    if ($@) {
-        run_exit_handlers();
-        die $@;
+    # Try extract.
+    $self->do_extract($newdirectory);
+
+    # Check for directory traversals.
+    if (not $self->{options}{skip_debianization} and not $self->{no_check}) {
+        # We need to add a trailing slash to handle the debian directory
+        # possibly being a symlink.
+        check_directory_traversal($newdirectory, "$newdirectory/debian/");
     }
 
-    # Store format if non-standard so that next build keeps the same format
-    if ($self->{fields}{'Format'} ne '1.0' and
+    # Store format if non-standard so that next build keeps the same format.
+    if ($self->{fields}{'Format'} and
+        $self->{fields}{'Format'} ne '1.0' and
         not $self->{options}{skip_debianization})
     {
         my $srcdir = File::Spec->catdir($newdirectory, 'debian', 'source');
         my $format_file = File::Spec->catfile($srcdir, 'format');
-	unless (-e $format_file) {
-	    mkdir($srcdir) unless -e $srcdir;
+        unless (-e $format_file) {
+            mkdir($srcdir) unless -e $srcdir;
             $self->{format}->save($format_file);
-	}
-    }
-
-    # Make sure debian/rules is executable
-    my $rules = File::Spec->catfile($newdirectory, 'debian', 'rules');
-    my @s = lstat($rules);
-    if (not scalar(@s)) {
-        unless ($! == ENOENT) {
-            syserr(g_('cannot stat %s'), $rules);
         }
-        warning(g_('%s does not exist'), $rules)
-            unless $self->{options}{skip_debianization};
-    } elsif (-f _) {
-        chmod($s[2] | 0111, $rules)
-            or syserr(g_('cannot make %s executable'), $rules);
-    } else {
-        warning(g_('%s is not a plain file'), $rules);
     }
 }
 
@@ -570,19 +658,16 @@ sub do_extract {
           'source package; use one of the subclasses';
 }
 
-# Function used specifically during creation of a source package
+# Function used specifically during creation of a source package.
 
 sub before_build {
     my ($self, $dir) = @_;
 }
 
 sub build {
-    my $self = shift;
-    eval { $self->do_build(@_) };
-    if ($@) {
-        run_exit_handlers();
-        die $@;
-    }
+    my ($self, @args) = @_;
+
+    $self->do_build(@args);
 }
 
 sub after_build {
@@ -607,16 +692,14 @@ sub add_file {
     }
     $self->{checksums}->add_from_file($filename, key => $fn);
     $self->{checksums}->export_to_control($self->{fields},
-					    use_files_for_md5 => 1);
+        use_files_for_md5 => 1,
+    );
 }
 
 sub commit {
-    my $self = shift;
-    eval { $self->do_commit(@_) };
-    if ($@) {
-        run_exit_handlers();
-        die $@;
-    }
+    my ($self, @args) = @_;
+
+    $self->do_commit(@args);
 }
 
 sub do_commit {
@@ -630,10 +713,10 @@ sub write_dsc {
     my $fields = $self->{fields};
 
     foreach my $f (keys %{$opts{override}}) {
-	$fields->{$f} = $opts{override}{$f};
+        $fields->{$f} = $opts{override}{$f};
     }
 
-    unless ($opts{nocheck}) {
+    unless ($opts{no_check}) {
         foreach my $f (qw(Source Version Architecture)) {
             unless (defined($fields->{$f})) {
                 error(g_('missing information for critical output field %s'), $f);
@@ -647,7 +730,7 @@ sub write_dsc {
     }
 
     foreach my $f (keys %{$opts{remove}}) {
-	delete $fields->{$f};
+        delete $fields->{$f};
     }
 
     my $filename = $opts{filename};
@@ -663,18 +746,42 @@ sub write_dsc {
 
 =head1 CHANGES
 
+=head2 Version 2.04 (dpkg 1.23.0)
+
+New options: certs, use_vendor_certs in $p->check_checksums().
+
+=head2 Version 2.03 (dpkg 1.22.7)
+
+New method: $p->get_basedirname().
+
+=head2 Version 2.02 (dpkg 1.21.10)
+
+New method: $p->armor_original_tarball_signature().
+
+=head2 Version 2.01 (dpkg 1.20.1)
+
+New method: $p->get_upstream_signing_key().
+
+=head2 Version 2.00 (dpkg 1.20.0)
+
+New method: $p->check_original_tarball_signature().
+
+Remove variable: $diff_ignore_default_regexp.
+
+Hide variable: @tar_ignore_default_pattern.
+
 =head2 Version 1.03 (dpkg 1.19.3)
 
 New option: format in new().
 
 =head2 Version 1.02 (dpkg 1.18.7)
 
-New option: require_strong_checksums in check_checksums().
+New option: require_strong_checksums in $p->check_checksums().
 
 =head2 Version 1.01 (dpkg 1.17.2)
 
-New functions: get_default_diff_ignore_regex(), set_default_diff_ignore_regex(),
-get_default_tar_ignore_pattern()
+New functions: $p->get_default_diff_ignore_regex(),
+$p->set_default_diff_ignore_regex(), $p->get_default_tar_ignore_pattern().
 
 Deprecated variables: $diff_ignore_default_regexp, @tar_ignore_default_pattern
 
